package com.xlht.xlhtproject.controller;

import com.alibaba.fastjson2.JSON;
import com.xlht.xlhtproject.domain.*;
import com.xlht.xlhtproject.mapper.*;
import com.xlht.xlhtproject.utils.CalculateUtils;
import com.xlht.xlhtproject.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

import static com.xlht.xlhtproject.enums.MyConstants.LONG_1;

@Component
@Slf4j
public class ScheduledTasks {


    @Autowired
    private TXlhtMedicineParamMapper tXlhtMedicineParamMapper;

    @Autowired
    private TShuju1Mapper shuju1Mapper;

    @Autowired
    private CalculateUtils calculateUtils;

    @Autowired
    private TTbDuMapper tTbDuMapper;

    @Autowired
    private TTbXieMapper tTbXieMapper;

    @Autowired
    private TXlhtAddRecordMapper xlhtAddRecordMapper;

    @Autowired
    private TXlhtConfMapper xlhtConfMapper;

    /*@Scheduled(fixedRate = 5, timeUnit = TimeUnit.MINUTES) // 五分钟行一次
    public void reportCurrentTime() {
        System.out.println("现在时间: " + System.currentTimeMillis() / 1000);
    }

    // 使用cron表达式
    @Scheduled(cron = "0 0/5 15,18 * * ?") // 每天的15点和18点，每隔5分钟执行一次
    public void fixedRateWithInitialDelay() {
        System.out.println("通过cron表达式执行的定时任务: " + System.currentTimeMillis() / 1000);
    }*/


    @Scheduled(fixedRate = 5, timeUnit = TimeUnit.MINUTES) // 五分钟行一次 修改数据库的值，跃渊获取然后自动投药
    public void addMedicine() {
        log.info("进入了定时投药的逻辑处理--开始");
        //计算一池和二池的值
        TXlhtMedicineParam latestRecord = this.tXlhtMedicineParamMapper.getLatestRecord();
        if(Objects.isNull(latestRecord)){
            log.error("锡林浩特定时投药处理：没有配置投药信息，无法投药");
            return;
        }
        //投药(写入数据库)
        //2024年09月29日16:10:17 因为鲍工那里组态不支持0，前端不做处理的情况下，新增的时候++，返回的时候--，其他就不用处理了
        this.parseData(latestRecord);
        //计算实时的碳源投加量 (2个池子需要分开存)
        BigDecimal val1 = this.calculateUtils.calculateMedicineByLastRecord(latestRecord, 1);
        if(!Objects.isNull(val1) && val1.compareTo(BigDecimal.ZERO) < 0){
            log.info("一池计算得到的碳源投加量是负数，按0处理");
            val1 = BigDecimal.ZERO;
        }
        BigDecimal val2 = this.calculateUtils.calculateMedicineByLastRecord(latestRecord, 2);
        if(!Objects.isNull(val2) && val2.compareTo(BigDecimal.ZERO) < 0){
            log.info("二池计算得到的碳源投加量是负数，按0处理");
            val2 = BigDecimal.ZERO;
        }
        TTbDu tTbDu = this.tTbDuMapper.selectTTbDuByID(LONG_1);
        if(Objects.isNull(tTbDu)){
            log.error("未查询到要写入的数据，初始化数据错误×××××××");
            return;
        }
        String now = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, DateUtils.getNowDate());
        tTbDu.setC1TjLl(val1);
        tTbDu.setC1TjLlTime(now);
        tTbDu.setC2TjLl(val2);
        tTbDu.setC2TjLlTime(now);
        //2024年10月13日17:05:42  除了更新表外，还要记录投药的数据
        TTbXie tTbXie = this.tTbXieMapper.selectTTbXieByID(LONG_1);
        if(!Objects.isNull(tTbXie)){
            TXlhtAddRecord xlhtAddRecord = TXlhtAddRecord.builder().build();
            latestRecord.setTytjTransientLLOne(tTbXie.getOneJYL());
            latestRecord.setTytjTransientLLTwo(tTbXie.getTwoJYL());

            Integer oneRemoteStatus = tTbXie.getOneRemoteStatus();
            Integer oneHLStatus = tTbXie.getOneHLStatus();
            Integer oneHandleStatus = tTbXie.getOneHandleStatus();
            if(!Objects.isNull(oneRemoteStatus) && oneRemoteStatus == 1 && !Objects.isNull(oneHLStatus) && oneHLStatus == 1 && !Objects.isNull(oneHandleStatus) && oneHandleStatus == 1){
                xlhtAddRecord.setOneSystemAdd(1);
            }else {
                xlhtAddRecord.setOneSystemAdd(0);
            }

            Integer twoRemoteStatus = tTbXie.getTwoRemoteStatus();
            Integer twoHLStatus = tTbXie.getTwoHLStatus();
            Integer twoHandleStatus = tTbXie.getTwoHandleStatus();
            if(!Objects.isNull(twoRemoteStatus) && twoRemoteStatus == 1 && !Objects.isNull(twoHLStatus) && twoHLStatus == 1 && !Objects.isNull(twoHandleStatus) && twoHandleStatus == 1){
                xlhtAddRecord.setTwoSystemAdd(1);
            }else{
                xlhtAddRecord.setTwoSystemAdd(0);
            }
            xlhtAddRecord.setOneAmount(val1);
            xlhtAddRecord.setTwoAmount(val2);
            xlhtAddRecord.setRealOneAmount(tTbXie.getOneJYL());
            xlhtAddRecord.setRealTwoAmount(tTbXie.getTwoJYL());
            xlhtAddRecord.setDelFlag(0L);
            Date nowDate = DateUtils.getNowDate();
            xlhtAddRecord.setCreateTime(nowDate);
            xlhtAddRecord.setRemark(DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, nowDate));
            this.xlhtAddRecordMapper.insertTXlhtAddRecord(xlhtAddRecord);
        }
        //2024年10月25日16:19:59 增加新的逻辑 记录组态返回的状态，然后下次判断
        TXlhtConf tXlhtConf = this.xlhtConfMapper.selectTXlhtConfById(LONG_1);
        if(!Objects.isNull(tXlhtConf)){//理论上不会存在空的
            //获取上次存取的值
            Integer lastOneStatus = tXlhtConf.getLastOneStatus();
            Integer lastTwoStatus = tXlhtConf.getLastTwoStatus();
            Integer addTypeOne = null;
            Integer addTypeTwo = null;
            //查询反馈的状态并更新到数据库中
            if(!Objects.isNull(tTbXie)){
                latestRecord.setTytjTransientLLOne(tTbXie.getOneJYL());
                latestRecord.setTytjTransientLLTwo(tTbXie.getTwoJYL());

                Integer oneRemoteStatus = tTbXie.getOneRemoteStatus();
                Integer oneHLStatus = tTbXie.getOneHLStatus();
                Integer oneHandleStatus = tTbXie.getOneHandleStatus();
                if(!Objects.isNull(oneRemoteStatus) && oneRemoteStatus == 1 && !Objects.isNull(oneHLStatus) && oneHLStatus == 1 && !Objects.isNull(oneHandleStatus) && oneHandleStatus == 1){
                    addTypeOne = 1;
                }else {
                    addTypeOne = 0;
                }

                Integer twoRemoteStatus = tTbXie.getTwoRemoteStatus();
                Integer twoHLStatus = tTbXie.getTwoHLStatus();
                Integer twoHandleStatus = tTbXie.getTwoHandleStatus();
                if(!Objects.isNull(twoRemoteStatus) && twoRemoteStatus == 1 && !Objects.isNull(twoHLStatus) && twoHLStatus == 1 && !Objects.isNull(twoHandleStatus) && twoHandleStatus == 1){
                    addTypeTwo = 1;
                }else{
                    addTypeTwo = 0;
                }
            }
            //如果不一致，则更新开关状态
            if(!Objects.isNull(addTypeOne) && !addTypeOne.equals(lastOneStatus)){
                log.info("11定时任务执行的时候，发现一池的组态返回的状态和上次返回的数据不一致，更新为关闭状态，组态返回数据为{}\n上次的一池状态为{}", JSON.toJSONString(tTbXie), lastOneStatus);
                tTbDu.setActiveOne(0L);
            }
            if(!Objects.isNull(addTypeTwo) && !addTypeTwo.equals(lastTwoStatus)){
                log.info("22定时任务执行的时候，发现二池的组态返回的状态和上次返回的数据不一致，更新为关闭状态，组态返回数据为{}\n上次的一池状态为{}", JSON.toJSONString(tTbXie), lastTwoStatus);
                tTbDu.setActiveTwo(0L);
            }
            //更新投加量、系统开关等
            this.tTbDuMapper.updateTTbDu(tTbDu);
            //更新组态返回的状态
            tXlhtConf.setLastOneStatus(addTypeOne);
            tXlhtConf.setLastTwoStatus(addTypeTwo);
            this.xlhtConfMapper.updateTXlhtConf(tXlhtConf);
        }

        log.info("进入了定时投药的逻辑处理--结束");
    }

    private void parseData(TXlhtMedicineParam latestRecord) {
        Integer typeOne = latestRecord.getTypeOne();
        if(!Objects.isNull(typeOne)){
            latestRecord.setTypeOne(typeOne - 1);
        }
        Integer typeTwo = latestRecord.getTypeTwo();
        if(!Objects.isNull(typeTwo)){
            latestRecord.setTypeTwo(typeTwo - 1);
        }
        Integer jsLlTypeOne = latestRecord.getJsLlTypeOne();
        if(!Objects.isNull(jsLlTypeOne)){
            latestRecord.setJsLlTypeOne(jsLlTypeOne - 1);
        }
        Integer jsLlTypeTwo = latestRecord.getJsLlTypeTwo();
        if(!Objects.isNull(jsLlTypeTwo)){
            latestRecord.setJsLlTypeTwo(jsLlTypeTwo - 1);
        }
        Integer jsCodTypeOne = latestRecord.getJsCodTypeOne();
        if(!Objects.isNull(jsCodTypeOne)){
            latestRecord.setJsCodTypeOne(jsCodTypeOne - 1);
        }
        Integer jsCodTypeTwo = latestRecord.getJsCodTypeTwo();
        if(!Objects.isNull(jsCodTypeTwo)){
            latestRecord.setJsCodTypeTwo(jsCodTypeTwo - 1);
        }
        Integer hycXsyTypeOne = latestRecord.getHycXsyTypeOne();
        if(!Objects.isNull(hycXsyTypeOne)){
            latestRecord.setHycXsyTypeOne(hycXsyTypeOne - 1);
        }
        Integer hycXsyTypeTwo = latestRecord.getHycXsyTypeTwo();
        if(!Objects.isNull(hycXsyTypeTwo)){
            latestRecord.setHycXsyTypeTwo(hycXsyTypeTwo - 1);
        }
        Integer qycYxyTypeOne = latestRecord.getQycYxyTypeOne();
        if(!Objects.isNull(qycYxyTypeOne)){
            latestRecord.setQycYxyTypeOne(qycYxyTypeOne - 1);
        }
        Integer qycYxyTypeTwo = latestRecord.getQycYxyTypeTwo();
        if(!Objects.isNull(qycYxyTypeTwo)){
            latestRecord.setQycYxyTypeTwo(qycYxyTypeTwo - 1);
        }
        Integer qycAdTypeOne = latestRecord.getQycAdTypeOne();
        if(!Objects.isNull(qycAdTypeOne)){
            latestRecord.setQycAdTypeOne(qycAdTypeOne - 1);
        }
        Integer qycAdTypeTwo = latestRecord.getQycAdTypeTwo();
        if(!Objects.isNull(qycAdTypeTwo)){
            latestRecord.setQycAdTypeTwo(qycAdTypeTwo - 1);
        }
        Integer jsTnTypeOne = latestRecord.getJsTnTypeOne();
        if(!Objects.isNull(jsTnTypeOne)){
            latestRecord.setJsTnTypeOne(jsTnTypeOne - 1);
        }
        Integer jsTnTypeTwo = latestRecord.getJsTnTypeTwo();
        if(!Objects.isNull(jsTnTypeTwo)){
            latestRecord.setJsTnTypeTwo(jsTnTypeTwo - 1);
        }
    }


}
